﻿using System;
using System.Collections.Generic;
using System.Drawing;

namespace MotionTracking
{
    public class EdgeCluster
    {
        //these hold the total of the x and y dimensions so that the mid point of a cluster can be 
        //determined
        private long sumX;
        private long sumY;

        List<EdgeData> edges = new List<EdgeData>();

        /// <summary>
        /// Add an edge and update the sum of x and y coordinates, which are used to determine where
        /// the average for all edges with this signature are.  Runs into problems when there is more
        /// than one object with the same signature
        /// </summary>
        /// <param name="newEdge"></param>
        public void AddEdge(EdgeData newEdge)
        {
            edges.Add(newEdge);
            sumX += newEdge.EdgeColumn;
            sumY += newEdge.EdgeRow;
        }

        public int NumberOfEdges()
        {
            return this.edges.Count;
        }

        public List<EdgeData> Edges { get { return edges; } }

        /// <summary>
        /// Returns center for all edges with a given signature
        /// </summary>
        /// <returns></returns>
        public Point GetClusterCenter()
        {
            return new Point((int)(sumX / this.edges.Count), (int)(sumY / this.edges.Count));
        }

    }
}
