﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;

namespace MotionTracking
{
    public class EdgeData : IComparable
    {
        //limit detection to significant shifts in color
        private const float redColorBand = 32.0f;
        private const float greenColorBand = 32.0f;
        private const float blueColorBand = 32.0f;

        public EdgeData(Color lastPixel, Color nextPixel, int row, int column)
        {
            Color lastLimitedColor = Color.FromArgb(255, ((int)(lastPixel.R /redColorBand) * (int)redColorBand),
                ((int)(lastPixel.G /greenColorBand) * (int)greenColorBand),
                ((int)(lastPixel.B / blueColorBand) * (int)blueColorBand));

            Color nextLimitedColor = Color.FromArgb(255, ((int)(nextPixel.R / redColorBand) * (int)redColorBand),
                ((int)(nextPixel.G / greenColorBand) * (int)greenColorBand),
                ((int)(nextPixel.B / blueColorBand) * (int)blueColorBand));

            this.HasEdge = false;

            this.PreEdge = lastLimitedColor;
            this.PostEdge = nextLimitedColor;

            if ((lastLimitedColor.R != nextLimitedColor.R) || (lastLimitedColor.G != nextLimitedColor.G) ||
                (lastLimitedColor.B != nextLimitedColor.B))
            {
                this.HasEdge = true;
            }

            this.EdgeColumn = column;
            this.EdgeRow = row;
        }

        public Color PreEdge { get; set; }

        public Color PostEdge { get; set; }

        public bool HasEdge { get; set; }

        public int EdgeRow { get; set; }

        public int EdgeColumn { get; set; }

        public virtual int CompareTo(object obj)
        {
            if (obj.GetType() != this.GetType())
            {
                throw new Exception("cannot compare something that isn't an instance of EdgeData with EdgeData");
            }
            EdgeData compareEdge = (EdgeData)obj;

            if ((this.PreEdge == compareEdge.PreEdge)&&(this.PostEdge == compareEdge.PostEdge))
            {
                return 0;
            }
            else if (this.PreEdge.ToArgb() > compareEdge.PreEdge.ToArgb())
            {
                return 1;
            }
            else if (this.PreEdge.ToArgb() < compareEdge.PreEdge.ToArgb())
            {
                return -1;
            }
            else if (this.PostEdge.ToArgb() > compareEdge.PostEdge.ToArgb())
            {
                return 1;
            }

            return -1;
            
        }
    }
}
