﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Media.Imaging;
using System.IO;

namespace MotionTracking
{
    public class EdgeMotionDetector : MotionDetector
    {
        SortedList<EdgeData, EdgeCluster> lastClusters = null;

        public override void AnalyzeNextImage(BitmapImage nextBitmapImage)
        {
            Bitmap nextBitmap = BitmapImage2Bitmap(nextBitmapImage);
            int imageWidth = nextBitmapImage.PixelWidth;
            int imageHeight = nextBitmapImage.PixelHeight;
            Color lastPixel = nextBitmap.GetPixel(0, 0);

            //both bitmaps must be the same width and height
            if ((imageWidth != nextBitmap.Width) || (imageHeight != nextBitmap.Height))
            {
                throw new Exception("last and current bitmap dimensions don't match");
            }

            ClusterManager clusterManager = new ClusterManager();
                    
            this.ResultBitmap = new Bitmap(imageWidth, imageHeight);
            
            //this first look adds all the edges found for the new image to a clustermanager
            //all edges are treated the same
            for (int rowCell = 0; rowCell < imageHeight; rowCell+=2)
            {
                for (int columnCell = 0; columnCell < imageWidth; columnCell+=2)
                {
                    Color pixelColor = nextBitmap.GetPixel(columnCell, rowCell);
                    if(columnCell!=0)
                    {
                        MonoChromeEdgeData edgeData = new MonoChromeEdgeData(lastPixel, pixelColor, rowCell, columnCell);
                        if (edgeData.HasEdge)
                        {
                            clusterManager.AddEdge(edgeData);
                        }
                    }
                    lastPixel = pixelColor;
                }
            }
           
            SortedList<EdgeData, EdgeCluster> currentClusters = clusterManager.GetClusters();

            if(lastClusters!=null)
            {
                foreach (EdgeData edgeData in currentClusters.Keys)
                {
                    EdgeCluster currentCluster = currentClusters[edgeData];

                    foreach (EdgeData newEdge in currentCluster.Edges)
                    {
                        this.ResultBitmap.SetPixel(newEdge.EdgeColumn - 1, newEdge.EdgeRow, newEdge.PreEdge);
                        this.ResultBitmap.SetPixel(newEdge.EdgeColumn, newEdge.EdgeRow, newEdge.PostEdge);
                    }
                }

                //remove all edge present in old frame
                foreach (EdgeData edgeData in lastClusters.Keys)
                {
                    EdgeCluster lastCluster = lastClusters[edgeData];

                    foreach (EdgeData oldEdge in lastCluster.Edges)
                    {
                        this.ResultBitmap.SetPixel(oldEdge.EdgeColumn - 1, oldEdge.EdgeRow, Color.White);
                        this.ResultBitmap.SetPixel(oldEdge.EdgeColumn, oldEdge.EdgeRow, Color.White);
                    }
                }
            }

            lastClusters= currentClusters;

        }

    }
}
