﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Media.Imaging;
using System.IO;

namespace MotionTracking
{
    public class EdgeSignatureMotionDetector : MotionDetector
    {
        private SortedList<EdgeData, EdgeCluster> lastClusters = null;

        public override void AnalyzeNextImage(BitmapImage nextBitmapImage)
        {
            Bitmap nextBitmap = BitmapImage2Bitmap(nextBitmapImage);
            int imageWidth = nextBitmapImage.PixelWidth;
            int imageHeight = nextBitmapImage.PixelHeight;
            Color lastPixel = nextBitmap.GetPixel(0, 0);

            ClusterManager clusterManager = new ClusterManager();

            this.ResultBitmap = new Bitmap(imageWidth, imageHeight);

            //this first look adds all the edges found for the new image to a clustermanager,
            //which stores them in a sortedlist keyed by color
            for (int rowCell = 0; rowCell < imageHeight; rowCell ++)
            {
                for (int columnCell = 0; columnCell < imageWidth; columnCell++)
                {
                    Color pixelColor = nextBitmap.GetPixel(columnCell, rowCell);
                    if (columnCell != 0)
                    {
                        EdgeData edgeData = new EdgeData(lastPixel, pixelColor, rowCell, columnCell);
                        if (edgeData.HasEdge)
                        {
                            clusterManager.AddEdge(edgeData);
                        }
                    }
                    lastPixel = pixelColor;
                }
            }

            SortedList<EdgeData, EdgeCluster> currentClusters = clusterManager.GetClusters();

            if (lastClusters != null)
            {
                //this compares the edges in the previous and current images to see if there
                //was motion
                foreach (EdgeData edgeData in currentClusters.Keys)
                {
                    EdgeCluster currentCluster = currentClusters[edgeData];
                    if (lastClusters.ContainsKey(edgeData))
                    {
                        EdgeCluster lastCluster = lastClusters[edgeData];
                        Point lastLocation = lastCluster.GetClusterCenter();
                        Point currentLocation = currentCluster.GetClusterCenter();

                        if ((Math.Abs(lastLocation.X - currentLocation.X) > (imageWidth/20.0)) ||
                            (Math.Abs(lastLocation.Y - currentLocation.Y) > (imageHeight /20.0)))
                        {

                            if (currentCluster.NumberOfEdges() > 50)
                            {
                                foreach (EdgeData pixelEdgeData in currentCluster.Edges)
                                {
                                    this.ResultBitmap.SetPixel(pixelEdgeData.EdgeColumn - 1, pixelEdgeData.EdgeRow, pixelEdgeData.PreEdge);
                                    this.ResultBitmap.SetPixel(pixelEdgeData.EdgeColumn, pixelEdgeData.EdgeRow, pixelEdgeData.PostEdge);
                                }
                            }
                        }
                    }
                }
            }

            lastClusters = currentClusters;
        }

    }
}
