﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

namespace MotionTracking
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        public MainWindow()
        {
            InitializeComponent();

            this.detectionTypeDropdown.Items.Add("Pixel Difference");
            this.detectionTypeDropdown.Items.Add("Edge Detection");
            this.detectionTypeDropdown.Items.Add("Color Signature Edge Detection");
            this.detectionTypeDropdown.SelectedIndex = 0;
        }

        private void startImageButton_Click(object sender, RoutedEventArgs e)
        {
            // Configure open file dialog box 
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.Filter = "Image Files (*.png, *.jpg)|*.png;*.jpg";
            dlg.DefaultExt = ".png"; // Default file extension 

            // Show open file dialog box 
            Nullable<bool> result = dlg.ShowDialog();

            // Process open file dialog box results 
            if (result == true)
            {
                // Open document 
                string fileName = dlg.FileName;
                // Do something with fileName  

                BitmapImage bmi = new BitmapImage(new Uri(fileName, UriKind.Absolute));
                this.imageStart.Source = bmi;
            }
        }

        private void finishImageButton_Click(object sender, RoutedEventArgs e)
        {
            // Configure open file dialog box 
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.Filter = "Image Files (*.png, *.jpg)|*.png;*.jpg";
            dlg.DefaultExt = ".png"; // Default file extension 

            // Show open file dialog box 
            Nullable<bool> result = dlg.ShowDialog();

            // Process open file dialog box results 
            if (result == true)
            {
                // Open document 
                string fileName = dlg.FileName;
                // Do something with fileName  

                BitmapImage bmi = new BitmapImage(new Uri(fileName, UriKind.Absolute));
                this.imageFinish.Source = bmi;
            }
        }

        private void quitButton_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void analyzeButton_Click(object sender, RoutedEventArgs e)
        {

        //private MotionDetector motionDetector = new MotionDetector();

            MotionDetector motionDetector = null;

            if (this.detectionTypeDropdown.SelectedIndex == 2)
            {
                motionDetector = new EdgeSignatureMotionDetector();
            }
            else if (this.detectionTypeDropdown.SelectedIndex == 1)
            {
                motionDetector = new EdgeMotionDetector();
            }
            else
            {
                motionDetector = new PixelChangeMotionDetector();
            }

            if ((this.imageStart.Source == null) || (this.imageStart.Source == null))
            {
                MessageBox.Show("Please set the two images");
                return;
            }

            try
            {
                motionDetector.AnalyzeNextImage((BitmapImage)this.imageStart.Source);
                motionDetector.AnalyzeNextImage((BitmapImage)this.imageFinish.Source);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Problem analyzing the images "+ex.ToString());
            }

            Bitmap resultBitmap = motionDetector.ResultBitmap;

            MemoryStream ms = new MemoryStream();
            resultBitmap.Save(ms, ImageFormat.Png);
            ms.Position = 0;
            BitmapImage bi = new BitmapImage();
            bi.BeginInit();
            bi.StreamSource = ms;
            bi.EndInit();
            this.imageResults.Source = bi; 

        }
    }
}
