﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Media.Imaging;
using System.IO;


namespace MotionTracking
{
    public class PixelChangeMotionDetector : MotionDetector 
    {

        Bitmap lastBitmap = null;
        int changeCutoff = 30;

        public override void AnalyzeNextImage(BitmapImage nextBitmapImage)
        {
            Bitmap nextBitmap = BitmapImage2Bitmap(nextBitmapImage);

            if (lastBitmap != null)
            {
                try
                {
                    int columns = lastBitmap.Width;
                    int rows = lastBitmap.Height;

                    //both bitmaps must be the same width and height
                    if ((columns != nextBitmap.Width) || (rows != nextBitmap.Height))
                    {
                        throw new Exception("last and current bitmap dimensions don't match");
                    }

                    this.ResultBitmap = new Bitmap(columns, rows);
                    for (int column = 0; column < columns; column++)
                    {
                        for (int row = 0; row < rows; row++)
                        {
                            Color lastColor = lastBitmap.GetPixel(column, row);
                            Color nextColor = nextBitmap.GetPixel(column, row);

                            //if any of the colors changed by more than the cutoff amount, it is 
                            //considered a change
                            if ((Math.Abs(lastColor.R - nextColor.R) > changeCutoff) ||
                                (Math.Abs(lastColor.G - nextColor.G) > changeCutoff) ||
                                (Math.Abs(lastColor.B - nextColor.B) > changeCutoff))
                            {
                                this.ResultBitmap.SetPixel(column, row, nextColor);
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine("problem analyzing motion " + ex.Message);
                }
            }
            lastBitmap = nextBitmap;
        }

    }
}
